@@ -18,7 +18,7 @@ from splendor.shader_library import ShaderLibrary
 import splendor.obj_mesh as obj_mesh
 from splendor.image import load_image, load_depth, validate_texture
 import splendor.json_numpy as json_numpy
-from splendor.exceptions import SplendorException
+from splendor.exceptions import SplendorException, SplendorEmptyMeshException
 from splendor.primitives import make_primitive
 
 max_num_lights = 8
@@ -49,9 +49,10 @@ class SplendorRender:
             ('direction_light', 'direction_lights'))
 
     def __init__(self,
-            assets=None,
-            default_view_matrix=None,
-            default_camera_projection=None):
+        assets=None,
+        default_view_matrix=None,
+        default_camera_projection=None,
+    ):
         """
         SplendorRender initialization
         
@@ -82,7 +83,7 @@ class SplendorRender:
         if default_camera_projection is None:
             default_camera_projection = default_default_camera_projection
         self.default_camera_projection = default_camera_projection
-
+        
         # scene data
         self.scene_description = {
             'meshes':{},
@@ -142,13 +143,18 @@ class SplendorRender:
         version = GL.glGetString(GL.GL_VERSION).decode('utf-8')
 
         GL.glEnable(GL.GL_DEPTH_TEST)
+        GL.glEnable(GL.GL_SCISSOR_TEST)
         GL.glEnable(GL.GL_TEXTURE_CUBE_MAP_SEAMLESS)
         GL.glDepthMask(GL.GL_TRUE)
         GL.glDepthFunc(GL.GL_LESS)
         GL.glDepthRange(0.0, 1.0)
 
         GL.glClearColor(0.,0.,0.,0.)
-
+    
+    def viewport_scissor(self, x, y, width, height):
+        GL.glViewport(x, y, width, height)
+        GL.glScissor(x, y, width, height)
+    
     # scene methods ============================================================
     
     def load_scene(self, scene, clear_scene=False, reload_assets=False):
@@ -165,6 +171,7 @@ class SplendorRender:
             Reload assets that exist both in the new scene data and the
             already loaded scene data (irrelevant if clear_scene=True)
         """
+        
         if clear_scene:
             self.clear_scene()
 
@@ -431,6 +438,8 @@ class SplendorRender:
         mesh_buffers = {}
 
         vertex_floats = numpy.array(mesh['vertices'], dtype=numpy.float32)
+        if len(vertex_floats) == 0:
+            raise SplendorEmptyMeshException
         if vertex_floats.shape[1] > 3:
             vertex_floats = vertex_floats[:,:3]
         normal_floats = numpy.array(mesh['normals'], dtype=numpy.float32)
@@ -475,6 +484,7 @@ class SplendorRender:
         Load a square mesh placed almost at the far clipping plane to render
         the background onto.
         """
+        
         if 'BACKGROUND' not in self.gl_data['mesh_buffers']:
             mesh_buffers = {}
             vertex_floats = numpy.array([
@@ -500,6 +510,7 @@ class SplendorRender:
         -----------
         name : str
         """
+        
         del(self.scene_description['meshes'][name])
         self.gl_data['mesh_buffers'][name]['vertex_buffer'].delete()
         self.gl_data['mesh_buffers'][name]['face_buffer'].delete()
@@ -611,7 +622,7 @@ class SplendorRender:
             The x,y focal length of the camera that captured the depthmap for
             reprojection into 3D
         """
-
+        
         if name in self.scene_description['depthmaps']:
             self.remove_depthmap(name)
 
@@ -670,6 +681,7 @@ class SplendorRender:
         -----------
         name : str
         """
+        
         del(self.scene_description['depthmaps'][name])
         self.gl_data['depthmap_buffers'][name]['depth_buffer'].delete()
         del(self.gl_data['depthmap_buffers'][name])
@@ -719,17 +731,19 @@ class SplendorRender:
     # image_light methods ======================================================
     
     def load_image_light(self,
-            name,
-            diffuse_cubemap,
-            reflect_cubemap,
-            offset_matrix = numpy.eye(4),
-            blur = 0.,
-            diffuse_gamma = 1.,
-            diffuse_bias = 0.,
-            reflect_gamma = 1.,
-            reflect_bias = 0.,
-            render_background = True,
-            set_active = False):
+        name,
+        diffuse_cubemap,
+        reflect_cubemap,
+        offset_matrix = numpy.eye(4),
+        blur = 0.,
+        diffuse_gamma = 1.,
+        diffuse_bias = 0.,
+        reflect_gamma = 1.,
+        reflect_bias = 0.,
+        render_background = True,
+        set_active = False,
+        lock_to_camera = False,
+    ):
         """
         Load an image light.
         
@@ -778,6 +792,7 @@ class SplendorRender:
         image_light_data['diffuse_bias'] = diffuse_bias
         image_light_data['reflect_gamma'] = reflect_gamma
         image_light_data['reflect_bias'] = reflect_bias
+        image_light_data['lock_to_camera'] = lock_to_camera
         self.scene_description['image_lights'][name] = image_light_data
         
         self.load_background_mesh()
@@ -1491,9 +1506,16 @@ class SplendorRender:
             A ditionary mapping instance names to integers.  Each integer will
             be assigned a unique color using the masks module.
         """
-        for instance_name, index in instance_indices.items():
-            instance_data = self.scene_description['instances'][instance_name]
-            instance_data['mask_color'] = masks.color_index_to_float(index)
+        indices = numpy.array(list(instance_indices.values()))
+        if indices.shape[0]:
+            colors = masks.color_index_to_float(indices)
+            for i, color in zip(instance_indices.keys(), colors):
+                instance_data = self.scene_description['instances'][i]
+                instance_data['mask_color'] = color
+            
+        #for instance_name, index in instance_indices.items():
+        #    instance_data = self.scene_description['instances'][instance_name]
+        #    instance_data['mask_color'] = masks.color_index_to_float(index)
 
     def set_instance_masks_to_mesh_indices(self, mesh_indices, instances=None):
         """
@@ -1755,11 +1777,13 @@ class SplendorRender:
     # color_render methods -----------------------------------------------------
     
     def color_render(self,
-            instances = None,
-            depthmap_instances = None,
-            flip_y = True,
-            clear = True,
-            finish = True):
+        instances = None,
+        depthmap_instances = None,
+        flip_y = True,
+        clear = True,
+        finish = True,
+        ignore_hidden = False,
+    ):
         """
         Renders instances and depthmap instances using the color program.
         
@@ -1780,6 +1804,7 @@ class SplendorRender:
         finish = True
             Whether or not to finish the frame using glFinish
         """
+        
         # clear
         if clear:
             self.clear_frame()
@@ -1852,7 +1877,7 @@ class SplendorRender:
         vertex_color_shader_instances = {}
         flat_color_shader_instances = {}
         for instance in instances:
-            if self.instance_hidden(instance):
+            if not ignore_hidden and self.instance_hidden(instance):
                 continue
             instance_material = self.get_instance_material_name(instance)
             instance_mesh = self.get_instance_mesh_name(instance)
@@ -1970,6 +1995,10 @@ class SplendorRender:
                             1, GL.GL_TRUE,
                             offset_matrix.astype(numpy.float32))
                     
+                    GL.glUniform1i(
+                            location_data['lock_image_light_to_camera'],
+                            image_light_data['lock_to_camera'])
+                    
                     image_light_properties = numpy.array([
                             image_light_data['diffuse_gamma'],
                             image_light_data['diffuse_bias'],
@@ -2206,9 +2235,12 @@ class SplendorRender:
 
         location_data = self.shader_library.get_shader_locations(
                 'background_shader')
-
+        
         # set the camera's view_matrix
-        view_matrix = self.scene_description['camera']['view_matrix']
+        if light_data['lock_to_camera']:
+            view_matrix = numpy.eye(4)
+        else:
+            view_matrix = self.scene_description['camera']['view_matrix']
         GL.glUniformMatrix4fv(
                 location_data['view_matrix'],
                 1, GL.GL_TRUE,
@@ -2270,7 +2302,14 @@ class SplendorRender:
 
     # mask_render methods ------------------------------------------------------
     
-    def mask_render(self, instances=None, flip_y=True, clear=True, finish=True):
+    def mask_render(
+        self,
+        instances=None,
+        flip_y=True,
+        clear=True,
+        finish=True,
+        ignore_hidden=False,
+    ):
         """
         Renders instances using the mask program.
         
@@ -2288,7 +2327,7 @@ class SplendorRender:
         finish = True
             Whether or not to finish the frame using glFinish
         """
-
+        
         # clear
         if clear:
             self.clear_frame()
@@ -2328,7 +2367,7 @@ class SplendorRender:
             # sort the instances
             mesh_instances = {}
             for instance in instances:
-                if self.instance_hidden(instance):
+                if not ignore_hidden and self.instance_hidden(instance):
                     continue
                 instance_mesh = self.get_instance_mesh_name(instance)
                 try:
@@ -2388,10 +2427,11 @@ class SplendorRender:
     # coord_render methods -----------------------------------------------------
     
     def coord_render(self,
-            instances=None,
-            flip_y=True,
-            clear=True,
-            finish=True,
+        instances=None,
+        flip_y=True,
+        clear=True,
+        finish=True,
+        ignore_hidden=False,
     ):
         """
         Renders instances using the coord program.
@@ -2445,6 +2485,8 @@ class SplendorRender:
             if instances is None:
                 instances = self.scene_description['instances']
             for instance_name in instances:
+                if not ignore_hidden and self.instance_hidden(instance_name):
+                    continue
                 self.coord_render_instance(instance_name)
 
         finally:
@@ -2466,9 +2508,8 @@ class SplendorRender:
             An optimization.  If the same mesh was rendered previously, there
             is no need to copy certain data to the GPU again.
         """
+        
         instance_data = self.scene_description['instances'][instance_name]
-        if instance_data['hidden']:
-            return
 
         instance_mesh = instance_data['mesh_name']
         coord_box = instance_data['coord_box']
